;***************************************************************************
;*
;* \file      utils.inc
;* \author    Wolfgang Neff
;* \version   1.0
;* \date      2016-10-25
;* \copyright (CC) BY-NO-NO
;*
;* \par Legend of parameters
;*      Rd: Destination register
;*      Rr: Source register 
;*      K: Constant data
;*      k: Constant address
;*      X: Indirect address register
;*
;* \par History
;*      Created: 2016-10-25
;*
;***************************************************************************

;** MACRO ******************************************************************
;*
;* delay_ms K
;*
;* Perform a delay.
;*
;* \param    K     The number of miliseconds to delay
;*
;* Performs a delay by busy wait.
;*  
;* Low registers used: None
;* High registers used: R16, R17, R18
;*
;* Note: The macro F_CPU is supposed to be defined to a constant defining
;* the CPU clock frequency in Hertz.
;*
;***************************************************************************
    .IFNDEF F_CPU
        .ERROR "F_CPU not defined."
    .ENDIF

    .MACRO delay_ms
    .IF @0 < 1
        .ERROR "Invalid delay."
    .ELIF BYTE4((F_CPU/1000*@0-4)/5)
        .ERROR "Delay is too long."
    .ELIF BYTE3((F_CPU/1000*@0-3)/4)
        ldi r16, BYTE1((F_CPU/1000*@0-4)/5)
        ldi r17, BYTE2((F_CPU/1000*@0-4)/5)
        ldi r18, BYTE3((F_CPU/1000*@0-4)/5)
loop:   subi r16, 0x01
        sbci r17, 0x00
        sbci r18, 0x00
        brne loop
    .ELSE
        ldi r16, BYTE1((F_CPU/1000*@0-3)/4)
        ldi r17, BYTE2((F_CPU/1000*@0-3)/4)
loop:   subi r16, 0x01
        sbci r17, 0x00
        brne loop
    .ENDIF
    .ENDMACRO

;** MACRO ******************************************************************
;*
;* delay_us K
;*
;* Perform a delay.
;*
;* \param    K     The number of microsecond to delay
;*
;* Performs a delay by busy wait.
;*  
;* Low registers used: None
;* High registers used: R16, R17
;*
;* Note: The macro F_CPU is supposed to be defined to a constant defining
;* the CPU clock frequency in Hertz.
;*
;***************************************************************************
    .MACRO delay_us
    .IF @0 < 1
        .ERROR "Invalid delay."
    .ELIF @0 < 10
        .ERROR "Delay is too short."
    .ELIF BYTE3((F_CPU/1000000*@0-3)/4)
        .ERROR "Delay is too long."
    .ELIF BYTE2((F_CPU/1000000*@0-2)/3)
        ldi r16, BYTE1((F_CPU/1000000*@0-3)/4)
        ldi r17, BYTE2((F_CPU/1000000*@0-3)/4)
loop:   subi r16, 0x01
        sbci r17, 0x00
        brne loop
    .ELSE
        ldi r16, BYTE1((F_CPU/1000000*@0-2)/3)
loop:   dec r16
        brne loop
    .ENDIF
    .ENDMACRO

;** MACRO ******************************************************************
;*
;* div Rr - R1:R0<-R1:R0/Rr, R2<-R1:R0%Rr
;*
;* Divide register pair R1:R0 by register Rr.
;*
;* \param       Rr   The register with the denominator
;* \param[in]   R0   Low byte of the numerator
;* \param[in]   R1   High byte of the numerator
;* \param[out]  R0   Low byte of the result
;* \param[out]  R1   High byte of the result
;* \param[out]  R2   The remainder of the division
;*
;* Divide register pair R1:R0 by register Rr. The result is stored in R1:R0,
;* the remainder in R2. Register Rr remains unchanged.
;*  
;* Low registers used: R3, R4
;* High registers used: None
;*
;* Note: Please refer to http://www.avr-asm-tutorial.net/avr_en/source/DIV8E.asm
;*
;***************************************************************************
    .MACRO div
        mov r3, r0
		mov r4, r1
        clr r2
        clr r1
        clr r0
        inc r0
loop:
        clc
        rol r3
        rol r4
        rol r2
        brcs cont1
        cp r2, @0
        brcs cont2
cont1:
        sub r2, @0
        sec
        rjmp cont3
cont2:
        clc
cont3:
        rol r0
        rol r1
        brcc loop
	.ENDMACRO
