;***************************************************************************
;*
;* \file      usart.inc
;* \author    Wolfgang Neff
;* \version   1.0
;* \date      2016-11-01
;* \copyright (CC) BY-NO-NO
;*
;* \par Baud rate settings for F_CPU = 2 MHz
;*        BAUD   BSEL   BSCALE
;*        9600     12      0
;*       14400    123     -4
;*       19200     11     -1
;*       38400      9     -2
;*       57600     75     -6
;*      115200     11     -7
;*
;* \par History
;*      Created: 2016-11-01
;*
;***************************************************************************

    .EQU USART_BSEL = 11
	.EQU USART_BSCALE = -7

;** MACRO ******************************************************************
;*
;* usart_init
;*
;* Initialize USART.
;*
;* Low registers used: None
;* High registers used: R16
;*
;***************************************************************************
    .MACRO usart_init
        lds r16, USB_USART_PORT_DIR
	    ori r16, USB_USART_TX_PIN_bm
        andi r16, ~USB_USART_RX_PIN_bm
        sts USB_USART_PORT_DIR, r16
        ldi r16, USART_CMODE_ASYNCHRONOUS_gc | USART_CHSIZE_8BIT_gc | USART_PMODE_DISABLED_gc
        sts USB_USART_MODULE_CTRLC, r16
        ldi r16, LOW(USART_BSEL)
        sts USB_USART_MODULE_BAUDCTRLA, r16 
        ldi r16, (-7<<USART_BSCALE_gp) | (0x0F & HIGH(USART_BSEL))
        sts USB_USART_MODULE_BAUDCTRLB, r16
        ldi r16, USART_RXEN_bm | USART_TXEN_bm
        sts USB_USART_MODULE_CTRLB, r16
    .ENDMACRO

;** MACRO ******************************************************************
;*
;* usart_getc Rd
;*
;* Get a character.
;*
;* \param    Rd    The destination register
;*
;* Read a character from the USART. CR is replaced by NL.
;*  
;* Low registers used: None
;* High registers used: None
;*
;* Note: This is a sychronous routine. It waits until data is available.
;*
;***************************************************************************
    .MACRO usart_getc
loop:
        lds @0, USB_USART_MODULE_STATUS
        sbrs @0, USART_RXCIF_bp
        rjmp loop
        lds @0, USB_USART_MODULE_DATA
		cpi @0, '\r'
		brne cont
		ldi @0, '\n'
cont:
    .ENDMACRO

;** MACRO ******************************************************************
;*
;* usart_putc Rr (16 <= r <= 31)
;*
;* Put a character.
;*
;* \param    Rr    The source register
;*
;* Write a character to USART. NL is replaced by CR NL.
;*  
;* Low registers used: R0
;* High registers used: None
;* I/O Memory used: GPIO0
;*
;* Note: This is a sychronous routine. It waits until the USART is clear
;* to send.
;*
;***************************************************************************
    .MACRO usart_putc
		cpi @0, '\n'
		brne loop2
        out GPIO_GPIOR0, @0
		ldi @0, '\r'
loop1:
        lds r0, USB_USART_MODULE_STATUS
        sbrs r0, USART_DREIF_bp
        rjmp loop1
        sts USB_USART_MODULE_DATA, @0
        in @0, GPIO_GPIOR0
loop2:
        lds r0, USB_USART_MODULE_STATUS
        sbrs r0, USART_DREIF_bp
        rjmp loop2
        sts USB_USART_MODULE_DATA, @0
    .ENDMACRO

;** MACRO ******************************************************************
;*
;* usart_puts k
;*
;* Put a string from SRAM.
;*
;* \param    k     The constant address of the string in SRAM
;*
;* Write a null terminated character string to the USART.
;*  
;* Low registers used: R0
;* High registers used: R16
;* Pointers used: X
;* I/O Memory used: GPIO0
;*
;* Note: This routine uses usart_putc to handle the individual characters.
;*
;***************************************************************************
    .MACRO usart_puts
	    ldi XL, low(@0)
        ldi XH, high(@0)
loop:
		ld r16, X+
		tst r16
		breq quit
		usart_putc r16
		rjmp loop
quit:
    .ENDMACRO

;** MACRO ******************************************************************
;*
;* usart_puts_p k
;*
;* Put a string from PROGMEM.
;*
;* \param    k     The constant address of the string in PROGMEM
;*
;* Write a null terminated character string to the USART.
;*  
;* Low registers used: R0
;* High registers used: R16
;* Pointers used: Z
;*
;* Note: This routine uses usart_putc to handle the individual characters.
;*
;***************************************************************************
    .MACRO usart_puts_p
	    ldi ZL, low(@0<<1)
        ldi ZH, high(@0<<1)
loop:
		lpm r16, Z+
		tst r16
		breq quit
		usart_putc r16
		rjmp loop
quit:
    .ENDMACRO

;** MACRO ******************************************************************
;*
;* usart_putn
;*
;* Put a decimal number.
;*
;* \param[in]   R0   Low byte of the number
;* \param[in]   R1   High byte of the number
;*
;* Writes a 16-bit number (r1:r0) to USART.
;*
;* Low registers used: R2, R3, R4
;* High registers used: R16, R17    
;* Pointers used: X
;* I/O Memory used: GPIO0, GPIO1, GPIO2, GPIO3, GPIO4
;*
;* Dependencies: convert.inc
;*
;***************************************************************************
    .MACRO usart_putn
	    rcall bin2asc16
		clr XH
		ldi XL, GPIO_GPIOR5
loop:   
        ld r16, -X
		cpi r16, ' '
		breq skip
		usart_putc r16
skip:
		cpi XL, GPIO_GPIOR0
        brne loop
    .ENDMACRO

